; interrupt driven IR receive 

_isrIn	movwf	Wisr	
	swapf	STATUS,W		
	clrf	STATUS	
	movwf	Sisr


	btfsc	INTCON, INTF
	goto	_irrxsm
	
	btfsc	INTCON, TMR0IF
	goto	_timerOut


_isrOut	swapf	Sisr,W	
	movwf	STATUS	
	swapf	Wisr,F	
	swapf	Wisr,W	
	retfie


_irrxsm	movfw	state
	addwf	PCL,F
	goto	_waitStart
	goto	_endStart
	goto	_waitData
	goto	_resetState

_waitStart	clrf	TMR0		; clear timer 0
	bank1
	bsf	OPTION_REG, INTEDG	; set for INT on rising edge
	bank0
	bsf	INTCON, TMR0IE		; enable TMR0 interrupt
	bcf	INTCON, TMR0IF		; clear TMR0 interrupt flag
	bcf	INTCON, INTF		; clear INT flag

	incf	state,F		; set to next state ( endStart)
	goto	_isrOut

_endStart	movfw	TMR0		; TMR0 value to W reg

	addlw	.215	; Test length of start pulse is 2.2ms < Pulse < 2.56ms
	bc	_errStart	; > 2.56mS error
	addlw	.5
	bnc	_errStart	; < 2.24mS error

	movlw	.177	; set up TMR0 to time out after 4.99mS
	movwf	TMR0	
	bcf	INTCON, TMR0IF	; clear TMR0 interrupt flag
	bcf	INTCON, INTF	; clear INTerrupt flag


	clrf	bitCount	; initialise variables for ir data receive
	clrf	irrxReg1
	clrf	irrxReg2
	clrf	irrxReg3
	incf	state,F	; set to next state ( waitData)
	goto	_isrOut



_waitData	movfw	TMR0	; save value in TMR0 to W reg

			; 20 bits requires 3 byte buffer to hold received data.
	setc		; Set Carry so irrxReg1, bit 0 will be set after shift first shift
	rlf	irrxReg1,F	; shift 3 byte ir receive buffer left	
	rlf	irrxReg2,F	; shift Carry into 2nd byte
	rlf	irrxReg3,F	; shift Carry into 3rd byte

	addlw	.47	; TMR0 + 47 > 255?
	skpnc		; skip if no carry
	goto	_errData	; >1.98mS to long, goto _errData
	
	addlw	.7	; TMR0 + (47 + 7) >255?
	skpnc		; skip on carry   ( >1.56mS )
	goto	_irrxOne	; received bit is a 1, don't need to test for 0 

	addlw	.8	; TMR0 + (47+7+8) < 256
	skpc		; skip on carry
	goto	_errData	; <1.02mS to short, goto _errData

	bcf	irrxReg1,0	; received bit is a 0, clear bit 0 of irrxReg1


_irrxOne	movlw	.177	; set up TMR0 to time out after 4.99mS
	movwf	TMR0
	bcf	INTCON, TMR0IF	; clear TMR0 interrupt flag
	bcf	INTCON, INTF	; clear INTerrupt flag


	btfss	irrxReg1,0	; toggle PORTA on/off according to bit received
	bcf	PORTA,1
	btfsc	irrxReg1,0
	bsf	PORTA,1	


	incf	bitCount,F	; increment bit counter
	goto	_isrOut
	


_errStart	movlw	'S'	; Load ASCII 'S' into W
	goto	_irrxerr
_errData	movlw	'D'	; Load ASCII 'D' into W

_irrxerr	clrf	TMR0	
	bcf	INTCON, TMR0IF
	bcf	INTCON, INTE
	bcf	INTCON, INTF

_irrSTX	call	_serialTX	; send ASCII char in W to SIO
	movlw	.10	
	call	_serialTX	; send LF
	movlw	.13
	call	_serialTX	; send CR

	movlw	.3	; set state to 3 (hold off)
	movwf	state
	goto	_isrOut	; exit interrupt



_timerOut	movf	state,W	; test state value
	addlw	-.2	; test if state 2 (waitData) 
	bnz	_resetState	; if not it was a timeout error, so goto reset state

_irrxDataEnd	movlw	-.12	; Check bitCount to see if we received valid number of bits
	addwf	bitCount,W	; did we receive 12 bits
	bz	_validLength	; yes
	addlw	-.3	; did we receive 15 bits
	bz	_validLength	; yes
	addlw	-.5	; did we receive 20 bits
	bnz	_resetState	; no, reset state
	
_validLength	movfw	irrxReg1	; buffer received data and bit count
	movwf	irrxBuf1	; so we can send it via SIO without next
	movfw	irrxReg2	; ir packet overwriting it
	movwf	irrxBuf2
	movfw	irrxReg3
	movwf	irrxBuf3
	movfw	bitCount
	movwf	bitCountBuf
	bsf	flags, irrxRdy	; set data ready flag

_resetState	bcf	PORTA,1	; turn off ir receive monitor LED
	clrf	state	; set state to 0 (waitStart)
	bcf	INTCON, TMR0IE	; clear TMR0 interrupt
	bank1
	bcf	OPTION_REG,INTEDG ; set INT to falling edge
	bank0
	bcf	INTCON, INTF	; clear INT flag
	bsf	INTCON, INTE	; enable INTerrupts
	goto	_isrOut



